/*
 * LatLongFormatException.java
 *
 * Created on 02 November 2007, 17:11
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package uk.co.mccombe.mapping;

/**
 * An exception class for errors in Latitude and Longitude formats
 * @author Mike
 */
public class LatLongFormatException extends java.lang.Exception {
    
    /**
     * Creates a new instance of <code>LatLongFormatException</code> without detail message.
     */
    public LatLongFormatException() {
    }
    /**
     * Constructs an instance of <code>LatLongFormatException</code> with the specified detail message.
     * @param msg the detail message.
     */
    public LatLongFormatException(String msg) {
        super(msg);
    }
}
