package uk.co.mccombe.mapping ;
/**
* An exception thrown when conversion from grid reference is impossible
*/
public class GridFormatException extends Exception {
/**
* Create an exception with the specified message
*   @param s text
*
*/
    public GridFormatException(String s) {
        super();
        val = new String(s) ;
    }
/**
* Convert Exception to a String
*
*   @return - the message
*/
    public String toString() {
        return "GridFormatException -- " + val ;
    }
    private String val ;
}