/**
 * MissingDataFileException -- thrown when we're unable to open a particular file
 * containing tile data. This happens if the user has failed to download the required tile
 * or has put it in the wrong place. Go gently here and try to be helpful!
 * 
 * @author Mike McCombe
 * 
 */

package uk.co.mccombe.terrain;

/**
 *
 * @author Mike
 */
public class MissingDataFileException extends Exception {
    /**
     * Constructs an instance of <code>MissingDataFileException</code> with the specified detail message.
     * @param msg the detail message.
     */
    public MissingDataFileException(String msg) {
        super(msg);
    }
}
