/**
 * DataFileException - Thrown after errors in handling height data files
 */
package uk.co.mccombe.terrain;

/**
 *
 * @author Mike
 */
public class DataFileException extends Exception{
    DataFileException(String msg){
        super(msg);
    }
}
