/*
 * DefaultsProperties.java
 *
 * Created on 12 May 2006, 10:59
 *
 */

package uk.co.mccombe.terrain;

/**
 *
 * @author Mike McCombe
 */
public class DefaultProperties extends java.util.Properties {
    
    /** Creates a new instance of DefaultProperties */
    public DefaultProperties() {
        super();
//        this.setProperty("region", "Eurasia");
        this.setProperty("demURL","https://e4ftl01.cr.usgs.gov/MEASURES/NASADEM_HGT.001/2000.02.11/");
        this.setProperty("autodownload", "false");
        this.setProperty("coordinatesystem", "OSGB");
        this.setProperty("currentgridref", "ST 100600");
        this.setProperty("example", "ST 700600");
        this.setProperty("ellipsoid", "Airy Sphere 1830");
        this.setProperty("datum", "Ordnance Survey of Great Britain 1936");
        this.setProperty("eastoffset", "0.0");
        this.setProperty("northoffset", "0.0");
        this.setProperty("heightoffset", "0.0");
        this.setProperty("e-w_range", "1000.00");
        this.setProperty("n-s_range", "1000.0");
        this.setProperty("spacing", "50.0");
        this.setProperty("alignment","0");
        this.setProperty("latitude","52.0");
        this.setProperty("longitude","-2.0");
        this.setProperty("locale", "GB");
        this.setProperty("useASTER", "nasa");
        this.setProperty("therionCoordinateSet","OSGB");
   }
}

